@@ -113,7 +113,7 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 		full_name, domain, name));
 	DEBUG(10, ("lookup_name: flags = 0x0%x\n", flags));
 
-	if (((flags & LOOKUP_NAME_DOMAIN) || (flags == 0)) &&
+	if ((flags & LOOKUP_NAME_DOMAIN) &&
 	    strequal(domain, get_global_sam_name()))
 	{
 
@@ -569,9 +569,10 @@ static bool lookup_rids(TALLOC_CTX *mem_ctx, const struct dom_sid *domain_sid,
 			const char ***names, enum lsa_SidType **types)
 {
 	int i;
+	struct dom_sid_buf buf;
 
 	DEBUG(10, ("lookup_rids called for domain sid '%s'\n",
-		   sid_string_dbg(domain_sid)));
+		   dom_sid_str_buf(domain_sid, &buf)));
 
 	if (num_rids) {
 		*names = talloc_zero_array(mem_ctx, const char *, num_rids);
@@ -788,6 +789,7 @@ static bool lookup_as_domain(const struct dom_sid *sid, TALLOC_CTX *mem_ctx,
 
 static bool check_dom_sid_to_level(const struct dom_sid *sid, int level)
 {
+	struct dom_sid_buf buf;
 	int ret = false;
 
 	switch(level) {
@@ -810,7 +812,8 @@ static bool check_dom_sid_to_level(const struct dom_sid *sid, int level)
 
 	DEBUG(10, ("%s SID %s in level %d\n",
 		   ret ? "Accepting" : "Rejecting",
-		   sid_string_dbg(sid), level));
+		   dom_sid_str_buf(sid, &buf),
+		   level));
 	return ret;
 }
 
@@ -1055,10 +1058,12 @@ bool lookup_sid(TALLOC_CTX *mem_ctx, const struct dom_sid *sid,
 {
 	struct lsa_dom_info *domain;
 	struct lsa_name_info *name;
+	struct dom_sid_buf buf;
 	TALLOC_CTX *tmp_ctx;
 	bool ret = false;
 
-	DEBUG(10, ("lookup_sid called for SID '%s'\n", sid_string_dbg(sid)));
+	DEBUG(10, ("lookup_sid called for SID '%s'\n",
+		   dom_sid_str_buf(sid, &buf)));
 
 	if (!(tmp_ctx = talloc_new(mem_ctx))) {
 		DEBUG(0, ("talloc_new failed\n"));
@@ -1092,10 +1097,12 @@ bool lookup_sid(TALLOC_CTX *mem_ctx, const struct dom_sid *sid,
 
  done:
 	if (ret) {
-		DEBUG(10, ("Sid %s -> %s\\%s(%d)\n", sid_string_dbg(sid),
+		DEBUG(10, ("Sid %s -> %s\\%s(%d)\n",
+			   dom_sid_str_buf(sid, &buf),
 			   domain->name, name->name, name->type));
 	} else {
-		DEBUG(10, ("failed to lookup sid %s\n", sid_string_dbg(sid)));
+		DEBUG(10, ("failed to lookup sid %s\n",
+			   dom_sid_str_buf(sid, &buf)));
 	}
 	TALLOC_FREE(tmp_ctx);
 	return ret;
@@ -1114,8 +1121,9 @@ static bool legacy_sid_to_unixid(const struct dom_sid *psid, struct unixid *id)
 	unbecome_root();
 
 	if (!ret) {
+		struct dom_sid_buf buf;
 		DEBUG(10,("LEGACY: mapping failed for sid %s\n",
-			  sid_string_dbg(psid)));
+			  dom_sid_str_buf(psid, &buf)));
 		return false;
 	}
 
@@ -1359,6 +1367,7 @@ bool sid_to_uid(const struct dom_sid *psid, uid_t *puid)
 	bool expired = true;
 	bool ret;
 	uint32_t rid;
+	struct dom_sid_buf buf;
 
 	/* Optimize for the Unix Users Domain
 	 * as the conversion is straightforward */
@@ -1367,8 +1376,9 @@ bool sid_to_uid(const struct dom_sid *psid, uid_t *puid)
 		*puid = uid;
 
 		/* return here, don't cache */
-		DEBUG(10,("sid %s -> uid %u\n", sid_string_dbg(psid),
-			(unsigned int)*puid ));
+		DEBUG(10,("sid %s -> uid %u\n",
+			  dom_sid_str_buf(psid, &buf),
+			  (unsigned int)*puid ));
 		return true;
 	}
 
@@ -1387,7 +1397,7 @@ bool sid_to_uid(const struct dom_sid *psid, uid_t *puid)
 		/* Not in cache. Ask winbindd. */
 		if (!winbind_sid_to_uid(puid, psid)) {
 			DEBUG(5, ("winbind failed to find a uid for sid %s\n",
-				  sid_string_dbg(psid)));
+				  dom_sid_str_buf(psid, &buf)));
 			/* winbind failed. do legacy */
 			return legacy_sid_to_uid(psid, puid);
 		}
@@ -1396,7 +1406,8 @@ bool sid_to_uid(const struct dom_sid *psid, uid_t *puid)
 	/* TODO: Here would be the place to allocate both a gid and a uid for
 	 * the SID in question */
 
-	DEBUG(10,("sid %s -> uid %u\n", sid_string_dbg(psid),
+	DEBUG(10,("sid %s -> uid %u\n",
+		  dom_sid_str_buf(psid, &buf),
 		(unsigned int)*puid ));
 
 	return true;
@@ -1412,6 +1423,7 @@ bool sid_to_gid(const struct dom_sid *psid, gid_t *pgid)
 	bool expired = true;
 	bool ret;
 	uint32_t rid;
+	struct dom_sid_buf buf;
 
 	/* Optimize for the Unix Groups Domain
 	 * as the conversion is straightforward */
@@ -1420,7 +1432,8 @@ bool sid_to_gid(const struct dom_sid *psid, gid_t *pgid)
 		*pgid = gid;
 
 		/* return here, don't cache */
-		DEBUG(10,("sid %s -> gid %u\n", sid_string_dbg(psid),
+		DEBUG(10,("sid %s -> gid %u\n",
+			  dom_sid_str_buf(psid, &buf),
 			(unsigned int)*pgid ));
 		return true;
 	}
@@ -1444,13 +1457,14 @@ bool sid_to_gid(const struct dom_sid *psid, gid_t *pgid)
 		if ( !winbind_sid_to_gid(pgid, psid) ) {
 
 			DEBUG(10,("winbind failed to find a gid for sid %s\n",
-				  sid_string_dbg(psid)));
+				  dom_sid_str_buf(psid, &buf)));
 			/* winbind failed. do legacy */
 			return legacy_sid_to_gid(psid, pgid);
 		}
 	}
 
-	DEBUG(10,("sid %s -> gid %u\n", sid_string_dbg(psid),
+	DEBUG(10,("sid %s -> gid %u\n",
+		  dom_sid_str_buf(psid, &buf),
 		  (unsigned int)*pgid ));
 
 	return true;
@@ -1551,9 +1565,11 @@ NTSTATUS get_primary_group_sid(TALLOC_CTX *mem_ctx,
 	if (need_lookup_sid) {
 		enum lsa_SidType type = SID_NAME_UNKNOWN;
 		bool lookup_ret;
+		struct dom_sid_buf buf;
 
 		DEBUG(10, ("do lookup_sid(%s) for group of user %s\n",
-			   sid_string_dbg(group_sid), username));
+			   dom_sid_str_buf(group_sid, &buf),
+			   username));
 
 		/* Now check that it's actually a domain group and
 		 * not something else */
@@ -1566,7 +1582,8 @@ NTSTATUS get_primary_group_sid(TALLOC_CTX *mem_ctx,
 
 		DEBUG(3, ("Primary group %s for user %s is"
 			  " a %s and not a domain group\n",
-			  sid_string_dbg(group_sid), username,
+			  dom_sid_str_buf(group_sid, &buf),
+			  username,
 			  sid_type_lookup(type)));
 	}
 
