@@ -71,7 +71,7 @@ static NTSTATUS samsync_ldb_add_foreignSecurityPrincipal(TALLOC_CTX *mem_ctx,
 							 struct ldb_dn **fsp_dn,
 							 char **error_string)
 {
-	const char *sidstr = dom_sid_string(mem_ctx, sid);
+	struct dom_sid_buf buf;
 	/* We assume that ForeignSecurityPrincipals are under the BASEDN of the main domain */
 	struct ldb_dn *basedn = samdb_search_dn(state->sam_ldb, mem_ctx,
 						state->base_dn[SAM_DATABASE_DOMAIN],
@@ -79,10 +79,6 @@ static NTSTATUS samsync_ldb_add_foreignSecurityPrincipal(TALLOC_CTX *mem_ctx,
 	struct ldb_message *msg;
 	int ret;
 
-	if (!sidstr) {
-		return NT_STATUS_NO_MEMORY;
-	}
-
 	if (basedn == NULL) {
 		*error_string = talloc_asprintf(mem_ctx, 
 						"Failed to find DN for "
@@ -98,7 +94,8 @@ static NTSTATUS samsync_ldb_add_foreignSecurityPrincipal(TALLOC_CTX *mem_ctx,
 
 	/* add core elements to the ldb_message for the alias */
 	msg->dn = basedn;
-	if ( ! ldb_dn_add_child_fmt(msg->dn, "CN=%s", sidstr))
+	if ( ! ldb_dn_add_child_fmt(
+		     msg->dn, "CN=%s", dom_sid_str_buf(sid, &buf)))
 		return NT_STATUS_UNSUCCESSFUL;
 	
 	ldb_msg_add_string(msg, "objectClass", "foreignSecurityPrincipal");
@@ -161,13 +158,22 @@ static NTSTATUS samsync_ldb_handle_domain(TALLOC_CTX *mem_ctx,
 			/* Update the domain sid with the incoming
 			 * domain (found on LSA pipe, database sid may
 			 * be random) */
-			samdb_msg_add_dom_sid(state->sam_ldb, mem_ctx, 
-					      msg, "objectSid", state->dom_sid[database]);
+			ret = samdb_msg_add_dom_sid(state->sam_ldb,
+						    mem_ctx,
+						    msg,
+						    "objectSid",
+						    state->dom_sid[database]);
+			if (ret != LDB_SUCCESS) {
+				return NT_STATUS_INTERNAL_ERROR;
+			}
 		} else {
 			/* Well, we will have to use the one from the database */
 			state->dom_sid[database] = samdb_search_dom_sid(state->sam_ldb, state,
 									state->base_dn[database], 
 									"objectSid", NULL);
+			if (state->dom_sid[database] == NULL) {
+				return NT_STATUS_INTERNAL_ERROR;
+			}
 		}
 
 		if (state->samsync_state->domain_guid) {
@@ -179,7 +185,10 @@ static NTSTATUS samsync_ldb_handle_domain(TALLOC_CTX *mem_ctx,
 				return status;
 			}
 			
-			ldb_msg_add_value(msg, "objectGUID", &v, NULL);
+			ret = ldb_msg_add_value(msg, "objectGUID", &v, NULL);
+			if (ret != LDB_SUCCESS) {
+				return NT_STATUS_INTERNAL_ERROR;
+			}
 		}
 	} else if (database == SAM_DATABASE_BUILTIN) {
 		/* work out the builtin_dn - useful for so many calls its worth
@@ -402,7 +411,12 @@ static NTSTATUS samsync_ldb_handle_user(TALLOC_CTX *mem_ctx,
 			samdb_msg_add_delete(state->sam_ldb, mem_ctx, msg,  
 					     remote_attrs[i]); 
 		} else {
-			ldb_msg_add(msg, el, LDB_FLAG_MOD_REPLACE);
+			ret = ldb_msg_add(msg, el, LDB_FLAG_MOD_REPLACE);
+			if (ret != LDB_SUCCESS) {
+				*error_string = talloc_strdup(
+					mem_ctx, "ldb_msg_add failed");
+				return NT_STATUS_NO_MEMORY;
+			}
 		}
 	}
 
@@ -653,7 +667,7 @@ static NTSTATUS samsync_ldb_handle_group_member(TALLOC_CTX *mem_ctx,
 						char **error_string) 
 {
 	uint32_t rid = delta->delta_id_union.rid;
-	struct netr_DELTA_GROUP_MEMBER *group_member = delta->delta_union.group_member;
+	struct netr_DELTA_GROUP_MEMBER *delta_group_member = delta->delta_union.group_member;
 	struct ldb_message *msg;
 	struct ldb_message **msgs;
 	int ret;
@@ -689,11 +703,11 @@ static NTSTATUS samsync_ldb_handle_group_member(TALLOC_CTX *mem_ctx,
 	
 	talloc_free(msgs);
 
-	for (i=0; i<group_member->num_rids; i++) {
+	for (i=0; i<delta_group_member->num_rids; i++) {
 		/* search for the group, by rid */
 		ret = gendb_search(state->sam_ldb, mem_ctx, state->base_dn[database], &msgs, attrs,
 				   "(&(objectClass=user)(objectSid=%s))", 
-				   ldap_encode_ndr_dom_sid(mem_ctx, dom_sid_add_rid(mem_ctx, state->dom_sid[database], group_member->rids[i]))); 
+				   ldap_encode_ndr_dom_sid(mem_ctx, dom_sid_add_rid(mem_ctx, state->dom_sid[database], delta_group_member->rids[i])));
 		
 		if (ret == -1) {
 			*error_string = talloc_asprintf(mem_ctx, "gendb_search failed: %s", ldb_errstring(state->sam_ldb));
@@ -956,21 +970,25 @@ static NTSTATUS samsync_ldb_handle_account(TALLOC_CTX *mem_ctx,
 	struct ldb_message *msg;
 	int ret;
 	uint32_t i;
-	char *dnstr, *sidstr;
+	char *dnstr;
+	struct dom_sid_buf buf;
 
 	msg = ldb_msg_new(mem_ctx);
 	if (msg == NULL) {
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	sidstr = dom_sid_string(msg, sid);
-	NT_STATUS_HAVE_NO_MEMORY_AND_FREE(sidstr, msg);
-
-	dnstr = talloc_asprintf(msg, "sid=%s", sidstr);
-	NT_STATUS_HAVE_NO_MEMORY_AND_FREE(dnstr, msg);
+	dnstr = talloc_asprintf(msg, "sid=%s", dom_sid_str_buf(sid, &buf));
+	if (dnstr == NULL) {
+		TALLOC_FREE(msg);
+		return NT_STATUS_NO_MEMORY;
+	}
 
 	msg->dn = ldb_dn_new(msg, state->pdb, dnstr);
-	NT_STATUS_HAVE_NO_MEMORY_AND_FREE(msg->dn, msg);
+	if (msg->dn == NULL) {
+		TALLOC_FREE(msg);
+		return NT_STATUS_NO_MEMORY;
+	}
 
 	for (i=0; i< account->privilege_entries; i++) {
 		ldb_msg_add_string(msg, "privilege", account->privilege_name[i].string);
@@ -1024,8 +1042,11 @@ static NTSTATUS samsync_ldb_delete_account(TALLOC_CTX *mem_ctx,
 	} else if (ret == 0) {
 		return NT_STATUS_NO_SUCH_USER;
 	} else if (ret > 1) {
-		*error_string = talloc_asprintf(mem_ctx, "More than one account with SID: %s", 
-						dom_sid_string(mem_ctx, sid));
+		struct dom_sid_buf buf;
+		*error_string = talloc_asprintf(
+			mem_ctx,
+			"More than one account with SID: %s",
+			dom_sid_str_buf(sid, &buf));
 		return NT_STATUS_INTERNAL_DB_CORRUPTION;
 	} else {
 		msg->dn = talloc_steal(msg, msgs[0]->dn);
@@ -1219,11 +1240,12 @@ NTSTATUS libnet_samsync_ldb(struct libnet_context *ctx, TALLOC_CTX *mem_ctx, str
 	state->secrets         = NULL;
 	state->trusted_domains = NULL;
 
-	state->sam_ldb         = samdb_connect(mem_ctx, 
+	state->sam_ldb         = samdb_connect(mem_ctx,
 					       ctx->event_ctx,
 					       ctx->lp_ctx,
 					       r->in.session_info,
-						   0);
+					       NULL,
+					       0);
 	if (!state->sam_ldb) {
 		return NT_STATUS_INTERNAL_DB_ERROR;
 	}
